import 'dart:async';
// ignore: avoid_web_libraries_in_flutter
import 'dart:html';
// ignore: avoid_web_libraries_in_flutter
import 'dart:js' as js;
import 'dart:convert';
import 'package:nui_miniprogram_core/nui_miniprogram_engine.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_enum.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_message.dart';

class NUIMiniProgramWebJSEngine with NUIMiniProgramJSEngineInterface{
  NUIMiniProgramWebJSEngine._builder();
  Window _window;

  static NUIMiniProgramWebJSEngine from(Window window){
    return NUIMiniProgramWebJSEngine._builder().._window = window;
  }

  void initCrossEventListener(NUICrossEventMessageListener listener){
    _window.addEventListener("message", (ev){
      MessageEvent messageEvent = ev;
      print("[NUIMiniProgramJSEngine] Message Event: ${messageEvent.data}, window name: ${window.name}");
      messageReceived(messageEvent.data);
    });
  }

  Future<bool> sendDefaultChannelEvent(NUIMiniProgramEventType type, {String payload}) async{
    final internalMessage = NUIMiniProgramInternalMessage(type: type, data: payload ?? type.value);
    final message = NUIMiniProgramMessage(channel: NUIMiniProgramMessage.DEFAULT_CHANNEL, data: jsonEncode(internalMessage));
    try {
      final result = await sendAndAwaitCrossEvent(message);
      return true;
    }catch(e){
      print("[NUIMiniProgramJSEngine] Failed to send ${type.value} event with error : $e");
      return false;
    }
  }

  Future<bool> sendPopEvent() async{
    final internalMessage = NUIMiniProgramInternalMessage(type: NUIMiniProgramEventType.POP, data: "pop");
    final popMessage = NUIMiniProgramMessage(channel: NUIMiniProgramMessage.DEFAULT_CHANNEL, data: jsonEncode(internalMessage));
    try {
      final result = await sendAndAwaitCrossEvent(popMessage);
      return true;
    }catch(e){
      print("[NUIMiniProgramJSEngine] Failed to send pop event with error : $e");
      return false;
    }
  }

  Future<bool> sendInitializedEvent() async{
    final internalMessage = NUIMiniProgramInternalMessage(type: NUIMiniProgramEventType.INITIALIZED, data: "initialized");
    final popMessage = NUIMiniProgramMessage(channel: NUIMiniProgramMessage.DEFAULT_CHANNEL, data: jsonEncode(internalMessage));
    try {
      final result = await sendAndAwaitCrossEvent(popMessage);
      return true;
    }catch(e){
      print("[NUIMiniProgramJSEngine] Failed to send initialized event with error : $e");
      return false;
    }
  }

  @override
  Future<bool> sendPlatformCrossEvent(NUIMiniProgramMessage message) async{
    try {
      final result = await js.context.callMethod('postNUIMessage', [jsonEncode(message)]);
      return true;
    }catch(e){
      print("[NUIMiniProgramJSEngine] Failed to post NUI Message with error : $e");
      return false;
    }
  }

}